﻿' EXAMPLE - SQL10 for Visual Basic using ADO.NET
' This example shows multi-user access to a database.  Two connections are
' established using shared mode.  In one data is inserted into the database.
' The other one also opens the database.  The second connection is used to
' read the data while the first connection still has a transaction open,
' which fails with a timeout.  Then the transaction is committed (freeing
' the locks) and the read using the second connection then succeeds.
' 
' The following methods are illustrated:
'     Raima.Rdm.RdmConnection:
'         BeginTransaction() Method
'         ChangeDatabase(string) Method
'         Close() Method
'         CreateCommand() Method
'         Open() Method
'         RdmConnection() Constructor
'     Raima.Rdm.RdmCommand:
'         CommandText Property
'         ExecuteNonQuery() Method
'         ExecuteReader() Method
'     Raima.Rdm.RdmDataReader:
'         Item[int] Property
'         Read() Method
'     Raima.Rdm.RdmTransaction
'         Commit() Method
' 
' They are fully document in http://docs.raima.com/rdm/.
' 
' For simplicity this example does not catch any thrown execptions
' but good programming practices would dictate that this occur in
' a full application.

' Raima.Rdm is the system module that contains the ADO.NET
' data provider for RDM
Imports Rdm = Raima.Rdm

Module SQL10
    Sub DropDatabase(conn As Rdm.RdmConnection, name As String)
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()

        ' Attempt to drop this database.  If the database does not already
        ' exist this will fail and we simply ignore the failure for this
        ' sample.
        Try
            cmd.CommandText = "DROP DATABASE " + name
            cmd.ExecuteNonQuery()
        Catch ex As Exception
        End Try

    End Sub

    Sub CreateThisDatabase(conn As Rdm.RdmConnection, name As String, cmds() As String)
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()

        cmd.CommandText = "CREATE DATABASE " + name
        cmd.ExecuteNonQuery()

        For Each c In cmds
            cmd.CommandText = c
            cmd.ExecuteNonQuery()
        Next

        cmd.CommandText = "COMMIT"
        cmd.ExecuteNonQuery()

        cmd.CommandText = "CLOSE " + name
        cmd.ExecuteNonQuery()
    End Sub

    Sub CreateDatabases(conn As Rdm.RdmConnection)
        ' Drop any existing copy of the databases.
        DropDatabase(conn, "SQL10")

        ' Attempt to create the database
        Dim cmds(0) As String
        cmds(0) = "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)"
        CreateThisDatabase(conn, "SQL10", cmds)
    End Sub

    Sub Main()
        ' Create the connection with a simple connection string
        Dim conn1 As Rdm.RdmConnection
        conn1 = New Rdm.RdmConnection("host=local;openmode=s")
        conn1.Open()

        CreateDatabases(conn1)

        ' Open the database
        conn1.ChangeDatabase("SQL10")

        ' Create a second connection
        Dim conn2 As Rdm.RdmConnection
        conn2 = New Rdm.RdmConnection("host=local;database=SQL10;openmode=s;timeout=2")
        conn2.Open()

        ' Create a command in connection 1 and one in connection 2
        Dim cmd1, cmd2 As Rdm.RdmCommand
        cmd1 = conn1.CreateCommand()
        cmd2 = conn2.CreateCommand()

        ' Start a new transaction in connection 1
        Dim trans As Rdm.RdmTransaction
        trans = conn1.BeginTransaction()

        ' Insert three rows into the table using connection 1
        For ii = 1 To 3
            cmd1.CommandText = "INSERT INTO Table1 VALUES " + Convert.ToString(ii)
            cmd1.ExecuteNonQuery()
        Next

        ' Attempt to read the data from connection 2; expect a timeout
        ' because connection 1 is holding locks as the transaction is still
        ' active
        Dim reader As Rdm.RdmDataReader
        Try
            cmd2.CommandText = "SELECT id FROM Table1"
            reader = cmd2.ExecuteReader()

            While reader.Read()
                Console.WriteLine("Should not get this data - " + Convert.ToString(reader.Item(0)))
            End While
        Catch ex As Exception
            ' Double-check to make sure the error is a timeout
            If ex.Message = "timeout expired" Then
                Console.WriteLine("Received expected timeout")
            Else
                Console.WriteLine("Unexpected exception occurred: " + Convert.ToString(ex))
            End If
        End Try

        ' Commit the transaction
        trans.Commit()

        ' Attempt to read the data again, which should now succeed
        cmd2.CommandText = "SELECT id FROM Table1"
        reader = cmd2.ExecuteReader()

        While (reader.Read())
            Console.WriteLine(Convert.ToString(reader.Item(0)))
        End While

        conn1.Close()
        conn2.Close()

        Console.WriteLine("Hit <Enter> to continue")
        Console.ReadLine()
    End Sub
End Module
